from os import path

from .info import __VERSION__

# <p>Copyright (c) 2005-2012 Stephen John Machin, Lingfo Pty Ltd</p>
# <p>This module is part of the xlrd package, which is released under a
# BSD-style licence.</p>

from . import licences

##
# <p><b>A Python module for extracting data from MS Excel (TM) spreadsheet files.
# <br /><br />
# Version 0.7.4 -- April 2012
# </b></p>
#
# <h2>General information</h2>
#
# <h3>Acknowledgements</h3>
#
# <p>
# Development of this module would not have been possible without the document
# "OpenOffice.org's Documentation of the Microsoft Excel File Format"
# ("OOo docs" for short).
# The latest version is available from OpenOffice.org in
# <a href=http://sc.openoffice.org/excelfileformat.pdf> PDF format</a>
# and
# <a href=http://sc.openoffice.org/excelfileformat.odt> ODT format.</a>
# Small portions of the OOo docs are reproduced in this
# document. A study of the OOo docs is recommended for those who wish a
# deeper understanding of the Excel file layout than the xlrd docs can provide.
# </p>
#
# <p>Backporting to Python 2.1 was partially funded by
#   <a href=http://journyx.com/>
#       Journyx - provider of timesheet and project accounting solutions.
#   </a>
# </p>
#
# <p>Provision of formatting information in version 0.6.1 was funded by
#   <a href=http://www.simplistix.co.uk>
#       Simplistix Ltd.
#   </a>
# </p>
#
# <h3>Unicode</h3>
#
# <p>This module presents all text strings as Python unicode objects.
# From Excel 97 onwards, text in Excel spreadsheets has been stored as Unicode.
# Older files (Excel 95 and earlier) don't keep strings in Unicode;
# a CODEPAGE record provides a codepage number (for example, 1252) which is
# used by xlrd to derive the encoding (for same example: "cp1252") which is
# used to translate to Unicode.</p>
# <small>
# <p>If the CODEPAGE record is missing (possible if the file was created
# by third-party software), xlrd will assume that the encoding is ascii, and keep going.
# If the actual encoding is not ascii, a UnicodeDecodeError exception will be raised and
# you will need to determine the encoding yourself, and tell xlrd:
# <pre>
#     book = xlrd.open_workbook(..., encoding_override="cp1252")
# </pre></p>
# <p>If the CODEPAGE record exists but is wrong (for example, the codepage
# number is 1251, but the strings are actually encoded in koi8_r),
# it can be overridden using the same mechanism.
# The supplied runxlrd.py has a corresponding command-line argument, which
# may be used for experimentation:
# <pre>
#     runxlrd.py -e koi8_r 3rows myfile.xls
# </pre></p>
# <p>The first place to look for an encoding ("codec name") is
# <a href=http://docs.python.org/lib/standard-encodings.html>
# the Python documentation</a>.
# </p>
# </small>
#
# <h3>Dates in Excel spreadsheets</h3>
#
# <p>In reality, there are no such things. What you have are floating point
# numbers and pious hope.
# There are several problems with Excel dates:</p>
#
# <p>(1) Dates are not stored as a separate data type; they are stored as
# floating point numbers and you have to rely on
# (a) the "number format" applied to them in Excel and/or
# (b) knowing which cells are supposed to have dates in them.
# This module helps with (a) by inspecting the
# format that has been applied to each number cell;
# if it appears to be a date format, the cell
# is classified as a date rather than a number. Feedback on this feature,
# especially from non-English-speaking locales, would be appreciated.</p>
#
# <p>(2) Excel for Windows stores dates by default as the number of
# days (or fraction thereof) since 1899-12-31T00:00:00. Excel for
# Macintosh uses a default start date of 1904-01-01T00:00:00. The date
# system can be changed in Excel on a per-workbook basis (for example:
# Tools -> Options -> Calculation, tick the "1904 date system" box).
# This is of course a bad idea if there are already dates in the
# workbook. There is no good reason to change it even if there are no
# dates in the workbook. Which date system is in use is recorded in the
# workbook. A workbook transported from Windows to Macintosh (or vice
# versa) will work correctly with the host Excel. When using this
# module's xldate_as_tuple function to convert numbers from a workbook,
# you must use the datemode attribute of the Book object. If you guess,
# or make a judgement depending on where you believe the workbook was
# created, you run the risk of being 1462 days out of kilter.</p>
#
# <p>Reference:
# http://support.microsoft.com/default.aspx?scid=KB;EN-US;q180162</p>
#
#
# <p>(3) The Excel implementation of the Windows-default 1900-based date system works on the
# incorrect premise that 1900 was a leap year. It interprets the number 60 as meaning 1900-02-29,
# which is not a valid date. Consequently any number less than 61 is ambiguous. Example: is 59 the
# result of 1900-02-28 entered directly, or is it 1900-03-01 minus 2 days? The OpenOffice.org Calc
# program "corrects" the Microsoft problem; entering 1900-02-27 causes the number 59 to be stored.
# Save as an XLS file, then open the file with Excel -- you'll see 1900-02-28 displayed.</p>
#
# <p>Reference: http://support.microsoft.com/default.aspx?scid=kb;en-us;214326</p>
#
# <p>(4) The Macintosh-default 1904-based date system counts 1904-01-02 as day 1 and 1904-01-01 as day zero.
# Thus any number such that (0.0 <= number < 1.0) is ambiguous. Is 0.625 a time of day (15:00:00),
# independent of the calendar,
# or should it be interpreted as an instant on a particular day (1904-01-01T15:00:00)?
# The xldate_* functions in this module
# take the view that such a number is a calendar-independent time of day (like Python's datetime.time type) for both
# date systems. This is consistent with more recent Microsoft documentation
# (for example, the help file for Excel 2002 which says that the first day
# in the 1904 date system is 1904-01-02).
#
# <p>(5) Usage of the Excel DATE() function may leave strange dates in a spreadsheet. Quoting the help file,
# in respect of the 1900 date system: "If year is between 0 (zero) and 1899 (inclusive),
# Excel adds that value to 1900 to calculate the year. For example, DATE(108,1,2) returns January 2, 2008 (1900+108)."
# This gimmick, semi-defensible only for arguments up to 99 and only in the pre-Y2K-awareness era,
# means that DATE(1899, 12, 31) is interpreted as 3799-12-31.</p>
#
# <p>For further information, please refer to the documentation for the xldate_* functions.</p>
#
# <h3> Named references, constants, formulas, and macros</h3>
#
# <p>
# A name is used to refer to a cell, a group of cells, a constant
# value, a formula, or a macro. Usually the scope of a name is global
# across the whole workbook. However it can be local to a worksheet.
# For example, if the sales figures are in different cells in
# different sheets, the user may define the name "Sales" in each
# sheet. There are built-in names, like "Print_Area" and
# "Print_Titles"; these two are naturally local to a sheet.
# </p><p>
# To inspect the names with a user interface like MS Excel, OOo Calc,
# or Gnumeric, click on Insert/Names/Define. This will show the global
# names, plus those local to the currently selected sheet.
# </p><p>
# A Book object provides two dictionaries (name_map and
# name_and_scope_map) and a list (name_obj_list) which allow various
# ways of accessing the Name objects. There is one Name object for
# each NAME record found in the workbook. Name objects have many
# attributes, several of which are relevant only when obj.macro is 1.
# </p><p>
# In the examples directory you will find namesdemo.xls which
# showcases the many different ways that names can be used, and
# xlrdnamesAPIdemo.py which offers 3 different queries for inspecting
# the names in your files, and shows how to extract whatever a name is
# referring to. There is currently one "convenience method",
# Name.cell(), which extracts the value in the case where the name
# refers to a single cell. More convenience methods are planned. The
# source code for Name.cell (in __init__.py) is an extra source of
# information on how the Name attributes hang together.
# </p>
#
# <p><i>Name information is <b>not</b> extracted from files older than
# Excel 5.0 (Book.biff_version < 50)</i></p>
#
# <h3>Formatting</h3>
#
# <h4>Introduction</h4>
#
# <p>This collection of features, new in xlrd version 0.6.1, is intended
# to provide the information needed to (1) display/render spreadsheet contents
# (say) on a screen or in a PDF file, and (2) copy spreadsheet data to another
# file without losing the ability to display/render it.</p>
#
# <h4>The Palette; Colour Indexes</h4>
#
# <p>A colour is represented in Excel as a (red, green, blue) ("RGB") tuple
# with each component in range(256). However it is not possible to access an
# unlimited number of colours; each spreadsheet is limited to a palette of 64 different
# colours (24 in Excel 3.0 and 4.0, 8 in Excel 2.0). Colours are referenced by an index
# ("colour index") into this palette.
#
# Colour indexes 0 to 7 represent 8 fixed built-in colours: black, white, red, green, blue,
# yellow, magenta, and cyan.<p>
#
# The remaining colours in the palette (8 to 63 in Excel 5.0 and later)
# can be changed by the user. In the Excel 2003 UI, Tools/Options/Color presents a palette
# of 7 rows of 8 colours. The last two rows are reserved for use in charts.<br />
# The correspondence between this grid and the assigned
# colour indexes is NOT left-to-right top-to-bottom.<br />
# Indexes 8 to 15 correspond to changeable
# parallels of the 8 fixed colours -- for example, index 7 is forever cyan;
# index 15 starts off being cyan but can be changed by the user.<br />
#
# The default colour for each index depends on the file version; tables of the defaults
# are available in the source code. If the user changes one or more colours,
# a PALETTE record appears in the XLS file -- it gives the RGB values for *all* changeable
# indexes.<br />
# Note that colours can be used in "number formats": "[CYAN]...." and "[COLOR8]...." refer
# to colour index 7; "[COLOR16]...." will produce cyan
# unless the user changes colour index 15 to something else.<br />
#
# <p>In addition, there are several "magic" colour indexes used by Excel:<br />
# 0x18 (BIFF3-BIFF4), 0x40 (BIFF5-BIFF8): System window text colour for border lines
# (used in XF, CF, and WINDOW2 records)<br />
# 0x19 (BIFF3-BIFF4), 0x41 (BIFF5-BIFF8): System window background colour for pattern background
# (used in XF and CF records )<br />
# 0x43: System face colour (dialogue background colour)<br />
# 0x4D: System window text colour for chart border lines<br />
# 0x4E: System window background colour for chart areas<br />
# 0x4F: Automatic colour for chart border lines (seems to be always Black)<br />
# 0x50: System ToolTip background colour (used in note objects)<br />
# 0x51: System ToolTip text colour (used in note objects)<br />
# 0x7FFF: System window text colour for fonts (used in FONT and CF records)<br />
# Note 0x7FFF appears to be the *default* colour index. It appears quite often in FONT
# records.<br />
#
# <h4>Default Formatting</h4>
#
# Default formatting is applied to all empty cells (those not described by a cell record).
# Firstly row default information (ROW record, Rowinfo class) is used if available.
# Failing that, column default information (COLINFO record, Colinfo class) is used if available.
# As a last resort the worksheet/workbook default cell format will be used; this
# should always be present in an Excel file,
# described by the XF record with the fixed index 15 (0-based). By default, it uses the
# worksheet/workbook default cell style, described by the very first XF record (index 0).
#
# <h4> Formatting features not included in xlrd version 0.6.1</h4>
# <ul>
#   <li>Rich text i.e. strings containing partial <b>bold</b> <i>italic</i>
#       and <u>underlined</u> text, change of font inside a string, etc.
#       See OOo docs s3.4 and s3.2.
#       <i> Rich text is included in version 0.7.2</i></li>
#   <li>Asian phonetic text (known as "ruby"), used for Japanese furigana. See OOo docs
#       s3.4.2 (p15)</li>
#   <li>Conditional formatting. See OOo docs
#       s5.12, s6.21 (CONDFMT record), s6.16 (CF record)</li>
#   <li>Miscellaneous sheet-level and book-level items e.g. printing layout, screen panes. </li>
#   <li>Modern Excel file versions don't keep most of the built-in
#       "number formats" in the file; Excel loads formats according to the
#       user's locale. Currently xlrd's emulation of this is limited to
#       a hard-wired table that applies to the US English locale. This may mean
#       that currency symbols, date order, thousands separator, decimals separator, etc
#       are inappropriate. Note that this does not affect users who are copying XLS
#       files, only those who are visually rendering cells.</li>
# </ul>
#
# <h3>Loading worksheets on demand</h3>
#
# <p>This feature, new in version 0.7.1, is governed by the on_demand argument
# to the open_workbook() function and allows saving memory and time by loading
# only those sheets that the caller is interested in, and releasing sheets
# when no longer required.</p>
#
# <p>on_demand=False (default): No change. open_workbook() loads global data
# and all sheets, releases resources no longer required (principally the
# str or mmap object containing the Workbook stream), and returns.</p>
#
# <p>on_demand=True and BIFF version < 5.0: A warning message is emitted,
# on_demand is recorded as False, and the old process is followed.</p>
#
# <p>on_demand=True and BIFF version >= 5.0: open_workbook() loads global
# data and returns without releasing resources. At this stage, the only
# information available about sheets is Book.nsheets and Book.sheet_names().</p>
#
# <p>Book.sheet_by_name() and Book.sheet_by_index() will load the requested
# sheet if it is not already loaded.</p>
#
# <p>Book.sheets() will load all/any unloaded sheets.</p>
#
# <p>The caller may save memory by calling
# Book.unload_sheet(sheet_name_or_index) when finished with the sheet.
# This applies irrespective of the state of on_demand.</p>
#
# <p>The caller may re-load an unloaded sheet by calling Book.sheet_by_xxxx()
#  -- except if those required resources have been released (which will
# have happened automatically when on_demand is false). This is the only
# case where an exception will be raised.</p>
#
# <p>The caller may query the state of a sheet:
# Book.sheet_loaded(sheet_name_or_index) -> a bool</p>
#
# <p> Book.release_resources() may used to save memory and close
# any memory-mapped file before proceding to examine already-loaded
# sheets. Once resources are released, no further sheets can be loaded.</p>
#
# <p> When using on-demand, it is advisable to ensure that
# Book.release_resources() is always called even if an exception
# is raised in your own code; otherwise if the input file has been
# memory-mapped, the mmap.mmap object will not be closed and you will
# not be able to access the physical file until your Python process
# terminates. This can be done by calling Book.release_resources()
# explicitly in the finally suite of a try/finally block.
# New in xlrd 0.7.2: the Book object is a "context manager", so if
# using Python 2.5 or later, you can wrap your code in a "with"
# statement.</p>
##

import sys, zipfile, pprint
from . import timemachine
from .biffh import (
    XLRDError,
    biff_text_from_num,
    error_text_from_code,
    XL_CELL_BLANK,
    XL_CELL_TEXT,
    XL_CELL_BOOLEAN,
    XL_CELL_ERROR,
    XL_CELL_EMPTY,
    XL_CELL_DATE,
    XL_CELL_NUMBER
    )
from .formula import * # is constrained by __all__
from .book import Book, colname #### TODO #### formula also has `colname` (restricted to 256 cols)
from .sheet import empty_cell
from .xldate import XLDateError, xldate_as_tuple

if sys.version.startswith("IronPython"):
    # print >> sys.stderr, "...importing encodings"
    import encodings

try:
    import mmap
    MMAP_AVAILABLE = 1
except ImportError:
    MMAP_AVAILABLE = 0
USE_MMAP = MMAP_AVAILABLE

##
#
# Open a spreadsheet file for data extraction.
#
# @param filename The path to the spreadsheet file to be opened.
#
# @param logfile An open file to which messages and diagnostics are written.
#
# @param verbosity Increases the volume of trace material written to the logfile.
#
# @param use_mmap Whether to use the mmap module is determined heuristically.
# Use this arg to override the result. Current heuristic: mmap is used if it exists.
#
# @param file_contents ... as a string or an mmap.mmap object or some other behave-alike object.
# If file_contents is supplied, filename will not be used, except (possibly) in messages.
#
# @param encoding_override Used to overcome missing or bad codepage information
# in older-version files. Refer to discussion in the <b>Unicode</b> section above.
# <br /> -- New in version 0.6.0
#
# @param formatting_info Governs provision of a reference to an XF (eXtended Format) object
# for each cell in the worksheet.
# <br /> Default is <i>False</i>. This is backwards compatible and saves memory.
# "Blank" cells (those with their own formatting information but no data) are treated as empty
# (by ignoring the file's BLANK and MULBLANK records).
# It cuts off any bottom "margin" of rows of empty (and blank) cells and
# any right "margin" of columns of empty (and blank) cells.
# Only cell_value and cell_type are available.
# <br /> <i>True</i> provides all cells, including empty and blank cells.
# XF information is available for each cell.
# <br /> -- New in version 0.6.1
#
# @param on_demand Governs whether sheets are all loaded initially or when demanded
# by the caller. Please refer back to the section "Loading worksheets on demand" for details.
# <br /> -- New in version 0.7.1
#
# @param ragged_rows False (the default) means all rows are padded out with empty cells so that all
# rows have the same size (Sheet.ncols). True means that there are no empty cells at the ends of rows.
# This can result in substantial memory savings if rows are of widely varying sizes. See also the
# Sheet.row_len() method.
# <br /> -- New in version 0.7.2
#
# @return An instance of the Book class.

def open_workbook(filename=None,
    logfile=sys.stdout,
    verbosity=0,
    use_mmap=USE_MMAP,
    file_contents=None,
    encoding_override=None,
    formatting_info=False,
    on_demand=False,
    ragged_rows=False,
    ):
    peeksz = 4
    if file_contents:
        peek = file_contents[:peeksz]
    else:
        f = open(filename, "rb")
        peek = f.read(peeksz)
        f.close()
    if peek == b"PK\x03\x04": # a ZIP file
        if file_contents:
            zf = zipfile.ZipFile(timemachine.BYTES_IO(file_contents))
        else:
            zf = zipfile.ZipFile(filename)
        component_names = zf.namelist()
        if verbosity:
            logfile.write('ZIP component_names:\n')
            pprint.pprint(component_names, logfile)
        if 'xl/workbook.xml' in component_names:
            from . import xlsx
            bk = xlsx.open_workbook_2007_xml(
                zf,
                component_names,
                logfile=logfile,
                verbosity=verbosity,
                use_mmap=use_mmap,
                formatting_info=formatting_info,
                on_demand=on_demand,
                ragged_rows=ragged_rows,
                )
            return bk
        if 'xl/workbook.bin' in component_names:
            raise XLRDError('Excel 2007 xlsb file; not supported')
        if 'content.xml' in component_names:
            raise XLRDError('Openoffice.org ODS file; not supported')
        raise XLRDError('ZIP file contents not a known type of workbook')

    from . import book
    bk = book.open_workbook_xls(
        filename=filename,
        logfile=logfile,
        verbosity=verbosity,
        use_mmap=use_mmap,
        file_contents=file_contents,
        encoding_override=encoding_override,
        formatting_info=formatting_info,
        on_demand=on_demand,
        ragged_rows=ragged_rows,
        )
    return bk

##
# For debugging: dump an XLS file's BIFF records in char & hex.
# @param filename The path to the file to be dumped.
# @param outfile An open file, to which the dump is written.
# @param unnumbered If true, omit offsets (for meaningful diffs).

def dump(filename, outfile=sys.stdout, unnumbered=False):
    from .biffh import biff_dump
    bk = Book()
    bk.biff2_8_load(filename=filename, logfile=outfile, )
    biff_dump(bk.mem, bk.base, bk.stream_len, 0, outfile, unnumbered)

##
# For debugging and analysis: summarise the file's BIFF records.
# I.e. produce a sorted file of (record_name, count).
# @param filename The path to the file to be summarised.
# @param outfile An open file, to which the summary is written.

def count_records(filename, outfile=sys.stdout):
    from .biffh import biff_count_records
    bk = Book()
    bk.biff2_8_load(filename=filename, logfile=outfile, )
    biff_count_records(bk.mem, bk.base, bk.stream_len, outfile)



#!/usr/home/rptlab2/bin/python -u
import fnmatch
import sys, os
from rlextra.utils.buildutils import kill

_debug = 0
_prune = ['(*)']

def find(pattern, dir = os.curdir, noDirs=0, noFiles=0):
	L = []
	names = os.listdir(dir)
	names.sort()
	for name in names:
		if name in (os.curdir, os.pardir): continue
		fullname = os.path.join(dir, name)
		isDir = os.path.isdir(fullname)
		if fnmatch.fnmatch(name, pattern):
			if (isDir and not noDirs) or (not isDir and not noFiles): L.append(fullname)
		if isDir and not os.path.islink(fullname):
			for p in _prune:
				if fnmatch.fnmatch(name, p):
					if _debug: print("skip", repr(fullname))
					break
			else:
				if _debug: print("descend into", repr(fullname))
				L += find(pattern, fullname, noDirs=noDirs, noFiles=noFiles)
	return L

def count_pages(infn):
	from rlextra.pageCatcher import pageCatcher
	from reportlab.pdfgen.canvas import Canvas
	from reportlab.lib import units
	import sys
	D = pageCatcher.storeFormsInDict(open(infn,'rb').read(),all=1)
	names = D[None]
	del D
	return len(names)

def main(indir):
	indir = os.path.normpath(os.path.abspath(indir))
	while indir[-1]==os.sep: indir = indir[:-1]
	if not os.path.isdir(indir): raise ValueError('%s isn\'t a directory' % indir)
	n = len(indir)
	m = n+len(os.sep)

	total = 0
	num = 0
	for fn in find('*.pdf', dir=indir, noDirs=1, noFiles=0):
		pc = count_pages(fn)
		total += pc
		num += 1
		print("%6d: '%s' %d pages, total=%d" % (num, fn[m:],pc,total))

if __name__=='__main__':
	if len(sys.argv)<2: raise ValueError('Need indir argument')
	for indir in sys.argv[1:]:
		main(indir)



#example6_twopage.py
#prints PDF two to a page.  Works best with A4 portrait files;

from reportlab.pdfgen import canvas
try:
    from rlextra.pageCatcher.pageCatcher import storeForms, restoreForms, twoPage
except ImportError:
    pass # running inside pageCatcher module?

def run():
    inputFileName = "sample2.pdf"
    outputFileName = "out6_twopage.pdf"
    storageFile = "sample2.data"

    print("2 up", inputFileName, "into", outputFileName)

    storeForms(inputFileName, storageFile, all=1)
    print("stored all pages to", storageFile)
    #canv = canvas.Canvas(outputFileName)
    #formNames = restoreForms(storageFile, canv)

    def twoPage(storagefile, testfile, scalefactor = 0.75):
        print("placing forms from", storagefile, "into", testfile, "two to a page")
        from reportlab.pdfgen import canvas
        canv = canvas.Canvas(testfile)
        (width, height) = canv._pagesize
        names = restoreForms(storagefile, canv, verbose=1)
        while names:
            for (xoff, yoff) in [ (0,2), (0,1) ]:
                thisname = names[0]
                print(thisname, end=' ')
                canv.saveState()
                (x,y) = (0, yoff*height/2.0)
                canv.translate(x,y)
                canv.rotate(-90)
                canv.scale(scalefactor, scalefactor)
                canv.doForm(thisname)
                canv.restoreState()
                del names[0]
                if not names: break
            canv.showPage()
            print()
        canv.save()
        print("wrote", testfile)

    twoPage(storageFile, outputFileName)

if __name__=='__main__':
    run()

    


def genPassword(minpairs, maxpairs,hackspell=0):
    """
    Generate pronounceable passwords, modified from: 
        http://www.zopelabs.com/cookbook/1059673251
    """
    import random

    if hackspell:
        vowels='4a3e1io0u'
        consonants='8bcdfg9hjklmnpqrs5t7vwxyz2'
    else:
        vowels='aeiou'
        consonants='bcdfghjkl1mnpqrst4vwxyz'
    password=''

    for x in range(1,random.randint(int(minpairs),int(maxpairs))+1):
         consonant = consonants[random.randint(1,len(consonants)-1)]
         password=password + consonant
         vowel = vowels[random.randint(1,len(vowels)-1)]
         password=password + vowel

    return password

if __name__=='__main__':
    from optparse import OptionParser

    parser = OptionParser("%prog [options]")
    parser.add_option("-n", "--minimum",
                      type="int",
                      dest="minimum",
                      default=3,
                      help="minimum password syllables(%default)")
    parser.add_option("-x", "--maximum",
                      type="int",
                      dest="maximum",
                      default=5,
                      help="maximum password syllables(%default)")
    parser.add_option("-H", "--hackspell",
                      type="int",
                      dest="hackspell",
                      default=0,
                      help="use hacker spelling length(%default)")

    opts, args = parser.parse_args()
    print(genPassword(opts.minimum,opts.maximum,opts.hackspell))



BASE2 = "01"
BASE10 = "0123456789"
BASE16 = "0123456789ABCDEF"
BASE36 = "0123456789abcdefghijklmnopqrstuvwxyz"
BASE62 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz"

def baseconvert(number,fromdigits,todigits):
    """ converts a "number" between two bases of arbitrary digits

    The input number is assumed to be a string of digits from the
    fromdigits string (which is in order of smallest to largest
    digit). The return value is a string of elements from todigits
    (ordered in the same way). The input and output bases are
    determined from the lengths of the digit strings. Negative 
    signs are passed through.

    decimal to binary
    >>> baseconvert(555,BASE10,BASE2)
    '1000101011'

    binary to decimal
    >>> baseconvert('1000101011',BASE2,BASE10)
    '555'

    integer interpreted as binary and converted to decimal (!)
    >>> baseconvert(1000101011,BASE2,BASE10)
    '555'

    base10 to base4
    >>> baseconvert(99,BASE10,"0123")
    '1203'

    base4 to base5 (with alphabetic digits)
    >>> baseconvert(1203,"0123","abcde")
    'dee'

    base5, alpha digits back to base 10
    >>> baseconvert('dee',"abcde",BASE10)
    '99'

    decimal to a base that uses A-Z0-9a-z for its digits
    >>> baseconvert(257938572394,BASE10,BASE62)
    'E78Lxik'

    ..convert back
    >>> baseconvert('E78Lxik',BASE62,BASE10)
    '257938572394'

    binary to a base with words for digits (the function cannot convert this back)
    >>> baseconvert('1101',BASE2,('Zero','One'))
    'OneOneZeroOne'
    >>> baseconvert(0,BASE10,BASE62)
    'A'
    >>> baseconvert('A',BASE62,BASE10)
    '0'
    >>> baseconvert('K',BASE62,BASE10)
    '10'
    >>> baseconvert(10,BASE10,BASE62)
    'K'
    """
    snum = str(number)
    neg = snum[0]=='-'
    if neg: snum = snum[1:]

    # make an integer out of the number
    x=0
    b = len(fromdigits)
    a = fromdigits.index
    for d in snum:
       x = x*b + a(d)
    
    # create the result in base 'len(todigits)'
    res = []
    a =res.append
    b = len(todigits)
    if not x: a(todigits[0])
    while x>0:
        x, d = divmod(x,b)
        a(todigits[d])
    if neg: a('-')
    res.reverse()

    return ''.join(res)

def _62_10(s):
    return baseconvert(s,BASE62,BASE10)

def _10_62(s):
    return baseconvert(s,BASE10,BASE62)

def _test():
    import doctest
    doctest.testmod()

if __name__ == "__main__":
    _test()




"convert play.dtd format into RML"

USEILLUSTRATIONS = 0
from rlextra.radxml.xmlmap import MapController, MapNode
from rlextra.rml2pdf import rml2pdf
import string

def annotation0(n, x1, y1, xn):
    L = ['<path x="%scm" y="%scm"> <curvesto>' %(x1, y1)]
    dx = (xn-x1)*1.0/n
    for i in range(n):
        for x in [(0,1), (0.5,1), (0.5,1),
                      (1,1), (1,0), (1,0),
                      (1, -1), (1.5, -1), (1.5, -1),
                      (2, -1), (2, 0), (2, 0)]:
            #print x
            (k,j) = x
            L.append("%scm %scm" % (x1+dx*(i+k/2.0), y1+(j/2.0)))
    L.append("</curvesto></path>")
    return string.join(L, "\n")

def annotationC(n, xc, yc, r, d):
    from math import pi, sin, cos
    angle = 2*pi/n
    L = ['<path x="%scm" y="%scm"> <curvesto>' % (xc+r, yc)]
    for i in range(n):
        baseangle = i*angle
        for (deltaangle, scale) in [(0, d), (1, d), (1, 1)]:
            anglen = baseangle + deltaangle*angle
            radius = r*scale
            x = cos(anglen)*radius + xc
            y = sin(anglen)*radius + yc
            L.append("%scm %scm" % (x,y))
    L.append("</curvesto></path>")
    return string.join(L, "\n")

def annotation1():
    xmin = ymin = 2
    xmax = 19
    ymax = 27
    L = [annotation0(xmax-xmin, xmin, ymin, xmax),
         annotation0(xmax-xmin, xmin, ymax, xmax),
         '<rotate degrees="90"/>',
         annotation0(ymax-ymin, ymin, -xmax, ymax),
         annotation0(ymax-ymin, ymin, -xmin, ymax),
         ]
    return string.join(L, "\n")

def annotation():
    return """
    <form name="annotation">
    %s
    </form>
    <lineMode width="3"/>
    %s
    <lineMode width="10"/>
    <stroke color="cyan"/>
    <doForm name="annotation"/>
    <translate dx="21cm" dy="29.7cm"/>
    <rotate degrees="180"/>
    <stroke color="lightcyan"/>
    <doForm name="annotation"/>
    <lineMode width="5"/>
    <translate dx="3" dy="3"/>
    <stroke color="blue"/>
    <doForm name="annotation"/>
    <translate dx="21cm" dy="29.7cm"/>
    <rotate degrees="180"/>
    <stroke color="black"/>
    <doForm name="annotation"/>
    """ % (annotation1(), annotationC(n=17, xc=10.5, yc=29.7/2, r=2, d=0.4))

TOPLEVELRML = """<?xml version="1.0" encoding="utf-8" standalone="no" ?>
<!DOCTYPE document SYSTEM "rml.dtd">

<document filename="[[PDFFILENAME]]">

<template pageSize="(21cm, 29.7cm)"
        leftMargin="1.5cm"
        rightMargin="1.5cm"
        topMargin="1.5cm"
        bottomMargin="1.5cm"
        title="Shakespeare"
        author="Reportlab Inc"
        allowSplitting = "20"
        >

	<pageTemplate id="titlepage">
	    <pageGraphics>
	    [[ANNOTATION]]
        </pageGraphics>

        <frame id="main" x1="5cm" y1="2cm" width="11cm" height="21.7cm"/>

	</pageTemplate>
	
	<pageTemplate id="main">
        <pageGraphics></pageGraphics>
        <pageGraphics>
        <fill color="crimson"/>
        <setFont name="Times-Roman" size="7"/>
        <drawRightString x="20cm" y="28cm"><getName id="header" default="???"/></drawRightString>
        <drawString x="1cm" y="28.5cm">http://www.reportlab.com</drawString>
        <!-- uncomment lastpage below to slow down generation 2x (but get page count) -->
        <drawString x="1cm" y="1cm"><getName id="playtitle" default="???"/>, page <pageNumber/><!-- of <getName id="lastpage" default="???"/> -->
        </drawString>
        </pageGraphics>
        <frame id="main" x1="3.5cm" y1="3.5cm" width="14cm" height="22.7cm"/>

	</pageTemplate>
</template>

<stylesheet>
	<initialize>
	</initialize>
	
	<paraStyle name="playtitle"
        fontSize="40"
        leading="50"
        alignment="center"
        spaceBefore="5cm"
        spaceAfter="3cm"
        textColor="darkslateblue"
	/>
	
	<paraStyle name="title"
        fontSize="20"
        leading="25"
        alignment="center"
        spaceBefore="2cm"
        spaceAfter="1cm"
        textColor="darkgreen"
	/>

	<paraStyle name="fm"
        fontSize="10"
        leading="12"
        alignment="center"
        spaceBefore="0"
        spaceAfter="0"
        textColor="darkmagenta"
	/>
	
	<paraStyle name="persona"
        fontSize="14"
        leading="18"
        alignment="center"
        spaceBefore="0"
        spaceAfter="0"
        textColor="darkmagenta"
	/>
	
	<paraStyle name="grpdescr"
        fontSize="14"
        leading="18"
        alignment="right"
        spaceBefore="0"
        spaceAfter="0"
        textColor="darkgreen"
	/>

	<paraStyle name="speaker"
        fontSize="14"
        fontName="Times-Bold"        
        leading="18"
        alignment="left"
        spaceBefore="0"
        spaceAfter="0"
        textColor="darkgreen"
	/>
	
	<paraStyle name="line"
        fontSize="12"
        leading="18"
        alignment="left"
        spaceBefore="0"
        spaceAfter="0"
        leftIndent="1in"
	/>
	
	<paraStyle name="stagedir"
        fontName="Times-Italic"
        fontSize="12"
        leading="18"
        alignment="left"
        spaceBefore="10"
        spaceAfter="10"
        leftIndent="2in"
        textColor="darkmagenta"
	/>

</stylesheet>
<story>
<setNextTemplate name="main"/>
%(__content__)s

<spacer length="2cm"/>
<h3>END OF <getName id="playtitle" default="???"/></h3>
<namedString id="lastpage"><pageNumber/></namedString>
</story>
</document>
"""

def translateFiles(inputfilename="policytext.xml", rmlfilename="shakespeare.rml", pdffilename="shakespeare.pdf"):
    fn = inputfilename
    ofn = rmlfilename
    text = open(fn).read()
    otext = translateText(text, outputfilename=pdffilename)
    open(ofn, "w").write(otext)
    print("wrote", ofn)

def translateText(text, outputfilename="shakespearetest.pdf"):
    # fix toplevel
    toplevel = string.replace(TOPLEVELRML, "[[PDFFILENAME]]", outputfilename)
    toplevel = string.replace(toplevel, "[[ANNOTATION]]", annotation())
    # now define the mapper
    M = MapController()
    M.joinSeparator = ""
    M[""] = """%(__content__)s"""
    M["PLAY"] = toplevel
    M["TITLE"] = """<para style="playtitle">
    <name id="playtitle" value="%(__content__)s"/>
    <name id="header" value="%(__content__)s"/>
    %(__content__)s</para>
    <spacer length="2in"/><para style="title">by William Shakespeare</para>"""
    M["SUBTITLE"] = """<h1>%(__content__)s</h1>"""
    M["SUBHEAD"] = """<h1>%(__content__)s</h1>"""
    M["FM"] = """<spacer length="0.2in"/>%(__content__)s"""
    M["FM"]["P"] = """<para style="fm">%(__content__)s</para>"""
    M["P"] = """<para>%(__content__)s</para>"""
    M["PERSONAE"] = """<nextFrame/>%(__content__)s"""
    M["PERSONAE"]["TITLE"] = """<para style="title">%(__content__)s</para>"""
    M["PERSONA"] = """<para style="persona">%(__content__)s</para>"""
    M["PGROUP"] = """<spacer length="1cm"/>%(__content__)s<spacer length="1cm"/>"""
    M["GRPDESCR"] = """<para style="grpdescr"><i>%(__content__)s</i></para>"""
    M["SCNDESCR"] = """<para style="title"><i>%(__content__)s</i></para>"""
    M["PLAYSUBT"] = """<para style="title">%(__content__)s</para>"""
    M["PROLOGUE"] = """<nextFrame/>%(__content__)s"""
    M["PROLOGUE"]["TITLE"] = """<para style="title">%(__content__)s</para>"""
    M["SPEECH"] = """<condPageBreak height="4cm"/><spacer length="1cm"/>%(__content__)s"""
    if not USEILLUSTRATIONS and 0:
        # non optimized
        print("using paragraphs for lines and speakers")
        M["SPEAKER"] = """<para style="speaker"><b>%(__content__)s</b></para>"""
        M["LINE"] = """<para style="line">%(__content__)s</para>"""
        M["LINE"]["STAGEDIR"] = """<font color="darkmagenta"><i>[%(__content__)s]</i></font>"""
    elif USEILLUSTRATIONS: # non optimized, we don't need paragraphs, preformatted is faster
        print("using illustrations for lines and speakers")
        M["SPEAKER"] = """<para style="speaker"><b>%(__content__)s</b></para>"""
        M["LINE"] = """<illustration width="0" height="14">
        <setFont name="Times-Roman" size="12"/>
        <fill color="black"/>
        <drawString x="1in" y="2">%(__content__)s</drawString>
        </illustration>
        """
        M["LINE"]["STAGEDIR"] = """</drawString></illustration>
    
        <illustration width="0" height="14">
        <setFont name="Times-Italic" size="12"/>
        <fill color="darkmagenta"/>
        <drawString x="2in" y="2">%(__content__)s</drawString>
        </illustration>
    
        <illustration width="0" height="14">
        <setFont name="Times-Roman" size="12"/>
        <fill color="black"/>
        <drawString x="1in" y="2">"""
    else:
        # optimized with preformatted text (30%+ faster) (slight format changes)
        print("using preformatted for lines and speakers")
        M["LINE"] = """<pre style="line">%(__content__)s</pre>"""
        M["LINE"]["STAGEDIR"] = """</pre><pre style="stagedir">
        [%(__content__)s]</pre> <pre style="line">"""
        M["SPEAKER"] = """<pre style="speaker">%(__content__)s</pre>"""
    M["STAGEDIR"] = """<para style="stagedir">%(__content__)s</para>"""
    M["ACT"] = """<nextFrame/>%(__content__)s"""
    M["ACT"]["TITLE"] = """<para style="title">%(__content__)s
    <name id="act" value="%(__content__)s"/>
    <name id="header" value="%(__content__)s"/></para>"""
    M["INDUCT"] = """<nextFrame/>%(__content__)s"""
    M["INDUCT"]["TITLE"] = """<para style="title">%(__content__)s
    <name id="act" value="%(__content__)s"/>
    <name id="header" value="%(__content__)s"/></para>"""
    M["EPILOGUE"] = """<nextFrame/>%(__content__)s"""
    M["EPILOGUE"]["TITLE"] = """<para style="title">%(__content__)s
    <name id="act" value="%(__content__)s"/>
    <name id="header" value="%(__content__)s"/></para>"""
    M["SCENE"] = """<spacer length="4cm"/><condPageBreak height="15cm"/>%(__content__)s"""
    M["SCENE"]["TITLE"] = """<para style="title">%(__content__)s
    <name id="scene" value="%(__content__)s"/></para>
    <namedString id="header"><getName id="act" default="???"/>, %(__content__)s</namedString>"""
    import pyRXPU
    from rlextra.radxml import xmlutils
    tree=pyRXPU.Parser().parse(text,ReturnUTF8=1)
    print('after tree parsed')
    tree = xmlutils.escapeTree(tree)
    print('after re entitising')
    otext = M.processParsed(tree)
    print('after processParsed')
    return otext

def test():
    from time import time
    from glob import glob
    files = glob("shakespeare.1.10.xml/*.xml")
    for file in files:
        now = time()
        rmlfile = file[:-4] + ".rml"
        pdffile = file[:-4] + ".pdf"
        print(); print("########## now processing", file, "into", rmlfile, "into", pdffile)
        translateFiles(inputfilename=file,
                       rmlfilename=rmlfile,
                       pdffilename=pdffile)
        text = open(rmlfile).read()
        rml2pdf.go(text)
        print("wrote", pdffile)

if __name__=="__main__":
    import sys
    from glob import glob
    def getFlag(name,nextVal=0):
        r = name in sys.argv
        if r:
            x = sys.argv.index(name)
            del sys.argv[x]
            if nextVal:
                r = sys.argv[x]
                del sys.argv[x]
        return r
    prof = getFlag('-prof')
    timing = getFlag('-timing')
    if timing:
        from time import time
        t0 = time()
        test()
        print('play2rml took %.2f seconds' % (time()-t0))
    elif prof:
        import profile
        profile.run('test()','play2rml.stats')
    else:
        test()



__code__='Gas1]D/U@^&>$Qs4,t7`f+!F#996ePNPrAATL&o-.bCdH3@:c\\#$jNmbKVBEk\'VC/>:7]FrKKp%iK*Iep_igkJ&*LE0##?4./*(B]fYgJ1ZNoWDj^E,Vth"Nln:,=\'3p(.-ia;)!8Cuc>QIa<1%8ZK4%KI8L\'_Moec>#d$\\/]%7DL8.3PZrV%fjoX?3"dB?3G\'H?34@9r"/?Qi(&;>"\'u$+"Ro@*X+8e%56JPn//MW]bk:J7X7.WZ>N"\'tei<hHJZQ\\m$0\'JWYkc.f(W:1nIa4q(Pf!qIF`%q\'1RKiBX:"#\\AQ]/$j\\c7k<lZ-[Y/rmu*D6_0)@#=TfUC&+"0?Kt"<7`Xr7V8_E(%TS&BZHfK,3!t=_=WB3&2V):q6pNk-4nPr\'WEdcZuuUDr\\snn7hp]USf-==P/bif3/msfkaMY<>b%^<EPZJ[L3Z`juD+8NNMtO:^p%\\T8nfi<%J"7<G0ib=m]fNkg$WJ\'-0PFGI!K%kbNufOeuAc`c3,iU+KOm[l0d6-%^m7`Vrtc?IADXEIsC1f1V_0C>q`2-`TaX-0q((AA_\\/(n/^4E6SW_;8-p4O_LQ^^u)FV#1#b^R+]1oKn51=M0>nT,g^_?4m0;TFo$NqHZ3r^W,a]b\\""QY4#ULW#J&=@pS3SK_^)Z(A]nd+.`)OUo^KU\\9LqGb"V(#a]gShkW[mYHrHl(\\q4Ho\\T&4pn^jW1;c:fUBXX$d.\'A)nCIUb##)<uRA<8g;qYF!2]VnSZBhg)CG*MTdeM$eG)9KElt5\'5-.]=W*M`r20@^U_,U*H+-bgSZH8Z\'Hm5,>+p79/$;h*:q9$%4<ltouGDFfof`hK=hrcb$Zl]GH)/$p7S%eX+"+L_sNE0#JEY\'o%Qo\\TY9`JO[K]U3H%qgaWB?\'kBVD"K8Bs.U&@)IU9[4+(!IJp?4"t%1]<KuoMUna`+o>g);.WCX>#8JLaHr(.MK^fVYP[N(hgM"/83!=[3H#5IHN\'rFg%%b2(#]Q\'O!i2pf6O6&uj>.(-Kp(,(_;,/]KAWTpF`?&:9,`="L[5kl9.11k%F3f8goCIDmLi9D(`Y.-bGmY8GHWDYsmM!G&D~>'



__all__ = ('solve',)
def solve(func, dfunc, target, x=1.0, precision=1e-008, maxIterations=100, failHard=1):
    """Finds x such that f(x)==target.
    Given an equation, 
        f(x) = 0
    and an initial approximation, x(0),
    in Newton's method a better approximation is given by: 
        x(i+1) = x(i) - f(x(i)) / f'(x(i))
    where f'(x) is the first derivative of f, df/dx.
    """
    iters = 0
    bf = None

    while iters<maxIterations:
        d = func(x)-target
        ad = abs(d)
        if bf is None or ad<bf:
            b = x
            bf = ad
        # if solved to RELATIVE precision, return now
        if ad/max(abs(target),1.0)<precision: return x
        x -= d/dfunc(x)
        iters += 1

    if failHard:
        raise ValueError("Failed to converge after %s tries\nbest abs error=%r at x=%r" % (maxIterations,bf,b))
    return b

def _test():
    def f(x):
        return x*x
    def df(x):
        return 2*x
    for target in [3,2,1]:
        x = solve(f,df,target)
        print('target=%f x=%f f(x)=%f' % (target,x,f(x)))

if __name__=='__main__': #noruntests
    _test()



#!/usr/bin/env python

"""Long document example generator"""

import glob
import re

from optparse import OptionParser

import preppy
from rlextra.rml2pdf import rml2pdf

regexp = {
    'chapter': re.compile("chapterName\s+\"(.*)\""),
    'section': re.compile("sectionName\s+\"(.*)\""),
    'appendix': re.compile("appendixName\s+\"(.*)\""),
    'part': re.compile("partName\s+\"(.*)\""),
    'slug': re.compile(r'[^A-Za-z0-9-]+')
    }


def parseCommandLine():
    """Examines options and does preliminary checking."""
    parser = OptionParser("run [options]")
    parser.add_option("--dull", action="store", dest="DULL", default=True, help="Dull or not dull ? ")
    # parser.add_option("-c", "--colour", action="store_true", dest="colour", default=True, help="Enable colour output.")
    # parser.add_option("-m", "--cropmarks", action="store_true", dest="cropmarks", default=False, help="Enable crop marks.")
    # parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="Print information about what is going on.")
    return parser.parse_args()

def prepareChapter(filename):

    FILE = open(filename, 'r')

    chapter = {
        'name': None,
        'id': None,
        'content': None,
        'part': None,
        'toc_sections': [],
        'sections': []
        }

    section = None
    toc_section = None
    content = ''

    
    while True:
        line = FILE.readline()
        if line:
            line = line.rstrip()
        else:
            if section:
                section['content'] = content
                chapter['sections'].append(section)
                chapter['toc_sections'].append(toc_section)
            elif content:
                chapter['content'] = content
            break

        is_part = regexp['part'].search(line)

        if is_part:
            line = ''
            chapter['part'] = {
                'id': regexp['slug'].sub('_', is_part.group(1)),
                'name': is_part.group(1)
                }

        if not chapter['name']:
            is_chapter = regexp['chapter'].search(line)
            if is_chapter:
                chapter['name'] = is_chapter.group(1)
                chapter['id'] = regexp['slug'].sub('_', chapter['name'])
        else:
            is_section = regexp['section'].search(line)
            if is_section:
                if section:
                    section['content'] = content
                    chapter['sections'].append(section)
                    chapter['toc_sections'].append(toc_section)
                else:
                    chapter['content'] = content
                section = {
                    'name': is_section.group(1),
                    'id': regexp['slug'].sub('_', is_section.group(1)),
                    'num': len(chapter['sections']) + 1
                    }
                toc_section = [section['name'], section['id'], section['num']]
                content = ''
            else:
                content += "%s\n" % line
    return chapter

def prepareAppendix(filename):
    FILE = open(filename, 'r')

    appendix = {
        'name': None,
        'id': None,
        'content': None,
        }

    content = ''
    
    while True:
        line = FILE.readline()
        if line:
            line = line.rstrip()
        else:
            break

        if not appendix['name']:
            is_appendix = regexp['appendix'].search(line)
            if is_appendix:
                appendix['name'] = is_appendix.group(1)
                appendix['id'] = regexp['slug'].sub('_', appendix['name'])
        else:
            content += "%s\n" % line


    appendix['content'] = content
    return appendix

    

def run(options):
    """Generate rml and pdf. """


    data = {
        'chapters_content': [],
        'chapters': [],
        'appendixes': [],
        'appendixes_content': [],
        }

    partInfo = None

    chapter_filenames = glob.glob("chapters/c_*prep")
    chapter_filenames.sort()
   
    for chapter_filename in chapter_filenames:
        chapter = prepareChapter(chapter_filename)
        chapter['num'] = len(data['chapters_content']) + 1
        data['chapters_content'].append(chapter)
        data['chapters'].append((chapter['num'], chapter['name'], chapter['part'], chapter['toc_sections']))

    appendix_filenames = glob.glob("chapters/a_*prep")
    appendix_filenames.sort()
    
    for appendix_filename in appendix_filenames:
        appendix = prepareAppendix(appendix_filename)
        appendix['num'] = len(data['appendixes_content'])
        data['appendixes_content'].append(appendix)
        data['appendixes'].append((appendix['name'], appendix['id']))


    template = preppy.getModule('rmltemplate')
    
    rmlText = template.get(data, options)
    rmlSource = open('rml-for-idiots.rml', 'w')
    rmlSource.writelines(rmlText)
    rmlSource.close()

    rml2pdf.go(rmlText, outputFileName='rml-for-idiots.pdf')

if __name__=='__main__':
    options, args = parseCommandLine()
    run(options)        
    print('created rml-for-idiots.pdf')

